//
//  Allocations.m - handles the user-controlled memory allocations
//  VMTester
//
//  Created by Cameron Hayne on 20/02/05.
//  Copyright 2006 Hayne of Tintagel.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#import "Allocations.h"
#import "AllocInfo.h"

#define BYTES_PER_MB (1024.0 * 1024.0)

// routines to make use of allocated memory so as to force it to become real
static void writeToMemory(char *ptr, size_t numBytes)
{
    // to make this faster, we only init every 10th byte
    for (int i = 0; i < numBytes; i += 10)
    {
        ptr[i] = i % 256;
    }
}

static int readFromMemory(char *ptr, size_t numBytes)
{
    float total = 0.0;
    // to make this faster, we only read every 10th byte
    for (int i = 0; i < numBytes; i += 10)
    {
        total += ptr[i];
    }
    return total;
}

@implementation Allocations

- (id)init
{
    myAllocInfoArray = [[NSMutableArray alloc] initWithCapacity: 100];
    return self;
}

- (void)dealloc
{
    [myAllocInfoArray release];
    [super dealloc];
}

#pragma mark Memory methods

- (BOOL)allocateMemory:(float)numMegaBytes Used:(bool)used
{
    int numBytes = (int) (BYTES_PER_MB * numMegaBytes + 0.5);
    void *ptr = malloc(numBytes);
    if (ptr == 0)
    {
        return NO;
    }
    
    if (used)
    {
        writeToMemory(ptr, numBytes);
    }
    
    AllocInfo *info = [[AllocInfo alloc] initWithNumBytes: numBytes
                                                  MemAddr: ptr
                                                  Used: used];
    [myAllocInfoArray addObject: info];
    return YES;
}

- (void)deallocateEntryAtIndex:(int)index
{
    AllocInfo *info = [myAllocInfoArray objectAtIndex: index];
    if (info != nil)
    {
        void *ptr = [info memAddr];
        free(ptr);
        
        [myAllocInfoArray removeObjectAtIndex: index];
    }
}

- (void)deallocateAll
{
    int numAllocs = [myAllocInfoArray count];
    for (int i = 0; i < numAllocs; i++)
    {
        AllocInfo *info = [myAllocInfoArray objectAtIndex: i];
        void *ptr = [info memAddr];
        free(ptr);
    }
    
    [myAllocInfoArray removeAllObjects];
}

- (double)readFromAllocatedMemory
{
    double total = 0.0;
    int numAllocs = [myAllocInfoArray count];
    for (int i = 0; i < numAllocs; i++)
    {
        AllocInfo *info = [myAllocInfoArray objectAtIndex: i];
        bool used = [info used];
        if (used == false)
        {
            continue;
        }
        void *ptr = [info memAddr];
        int numBytes = [info numBytes];
        total += readFromMemory(ptr, numBytes);
    }
    return total;
}

#pragma mark Accessors

- (NSArray *)allocInfoArray
{
    return myAllocInfoArray;
}

- (unsigned)numAllocs
{
    return [myAllocInfoArray count];
}

@end
