//
//  HTRingBuffer.m - utility that provides a "ring buffer"
//  VMTester
//
//  Created by Cameron Hayne on Tue Aug 19 2003.
//  Copyright 2006 Hayne of Tintagel.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#import "HTRingBuffer.h"

@implementation HTRingBuffer

- (id)initWithCapacity:(unsigned)maxItems
{
    self = [super init];
    if (self == nil)
        return nil;

    myCapacity = maxItems;
    myCurrentIndex = -1;
    myObjects = [[NSMutableArray arrayWithCapacity: maxItems] retain];

    return self;
}

- (void)dealloc
{
    [myObjects release];
    [super dealloc];
}

- (void)setCapacity:(unsigned)maxItems
{
    NSMutableArray *newObjects = [[NSMutableArray arrayWithCapacity: maxItems] retain];

    if (myObjects != nil)
    {
        unsigned count = [myObjects count];
        if (count > maxItems)
        {
            unsigned numExtra = count - maxItems;
            NSRange extra = NSMakeRange(0, numExtra - 1);
            [myObjects removeObjectsInRange: extra];
        }
        [newObjects addObjectsFromArray: myObjects];
        [myObjects release];
    }

    myObjects = newObjects;
    myCapacity = maxItems;
}

- (unsigned)capacity
{
    return myCapacity;
}

- (unsigned)count
{
    return [myObjects count];
}

- (void)addObject:(id)object
{
    if ([self count] >= myCapacity)
    {
        if (myCurrentIndex == 0)
        {
            // we would have to lose the current object if we added the new one
            // and we don't want to do that so we just return
            return;
        }

        [myObjects removeObjectAtIndex: 0];
        if (myCurrentIndex > 0)
        {
            // all the objects have moved down by one
            --myCurrentIndex;
        }

    }
    [myObjects addObject: object];
}

- (id)firstObject
{
    return [myObjects objectAtIndex: 0];
}

- (id)lastObject
{
    return [myObjects lastObject];
}

- (id)objectAtIndex:(unsigned)index
{
    return [myObjects objectAtIndex: index];
}

- (void)setCurrentIndex:(unsigned)index
{
    myCurrentIndex = index;
}

- (void)setCurrentIndexToNone
{
    [self setCurrentIndex: -1];
}

- (void)setCurrentIndexToLast
{
    int lastIndex = [self count] - 1;
    [self setCurrentIndex: lastIndex];
}

- (void)setCurrentIndexToPrevious
{
    if (myCurrentIndex > 0)
    {
        --myCurrentIndex;
    }
}

- (void)setCurrentIndexToNext
{
    if (myCurrentIndex < ([self count] - 1))
    {
        ++myCurrentIndex;
    }
}

- (void)setCurrentIndexBackBy:(int)num
{
    if (num <= 0)
    {
        // this indicates a programming error on the part of the caller
        return;
    }

    if (myCurrentIndex >= num)
    {
        myCurrentIndex -= num;
    }
    else // set to first
    {
        myCurrentIndex = 0;
    }
}

- (void)setCurrentIndexForwardBy:(int)num
{
    if (num <= 0)
    {
        // this indicates a programming error on the part of the caller
        return;
    }

    if (myCurrentIndex < ([self count] - num))
    {
        myCurrentIndex += num;
    }
    else // set to last
    {
        myCurrentIndex = [self count] - 1;
    }
}

- (id)currentObject
{
    if (myCurrentIndex >= 0 && myCurrentIndex < [self count])
    {
        return [myObjects objectAtIndex: myCurrentIndex];
    }
    return nil;
}

- (id)previousObject
{
    if (myCurrentIndex > 0 && myCurrentIndex < [self count])
    {
        return [myObjects objectAtIndex: (myCurrentIndex - 1)];
    }
    return nil;
}

@end
